/*
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Hewlett-Packard Company makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 */

/*
The Safe STL enhancements are

Copyright (C) Horstmann Software Design Corp. 1995. All Rights Reserved.

You may freely use, copy, modify, or distribute these Safe STL enhancements
and this documentation for any purpose, provided (1) you charge NO FEE for
distributing these copyrighted materials and (2) you include both the
copyright notice and permission notice from Hewlett-Packard Company and this
copyright notice and permission notice. Horstmann Software Design Corporation
specifically disclaims that these materials might do anything useful at all.
If they work for you and don't destroy anything, you are in luck.
There is no free technical support available for these materials.

I marked all modifications to the STL source with CSH (Cay S. Horstmann)

Please send bug reports/suggestions to horstman@cs.sjsu.edu.
For the most current version of these files, see
http://www.mathcs.sjsu.edu/faculty/horstman/safestl.html
*/


#ifndef LIST_H
#define LIST_H

#include <function.h>
// CSH #include <algobase.h>
#include <algorith> // CSH
#include <iterator.h>
// CSH #include <bool.h>
// CSH begin
#ifndef assert
#include <assert.h>
#endif
// CSH end

// CSH begin
namespace std {
// CSH end

#ifndef Allocator
#define Allocator allocator
// CSH #include <defalloc.h>
#include <memory> // CSH
#endif

#ifndef list
#define list list
#endif

template <class T>
class list {
protected:
	 typedef Allocator<void>::pointer void_pointer;
	 struct list_node;
	 friend list_node;
    struct list_node {
	void_pointer next;
	void_pointer prev;
	T data;
    };
    static Allocator<list_node> list_node_allocator;
    static Allocator<T> value_allocator;
public:      
    typedef T value_type;
    typedef Allocator<T> value_allocator_type;
    typedef Allocator<T>::pointer pointer;
    typedef Allocator<T>::reference reference;
    typedef Allocator<T>::const_reference const_reference;
	 typedef Allocator<list_node> list_node_allocator_type;
    typedef Allocator<list_node>::pointer link_type;
    typedef Allocator<list_node>::size_type size_type;
    typedef Allocator<list_node>::difference_type difference_type;
protected:
	 size_type buffer_size() {
	return list_node_allocator.init_page_size();
    }
    struct list_node_buffer;
    friend list_node_buffer;
    struct list_node_buffer {
	void_pointer next_buffer;
	link_type buffer;
    };
public:
    typedef Allocator<list_node_buffer> buffer_allocator_type;
    typedef Allocator<list_node_buffer>::pointer buffer_pointer;     
protected:
    static Allocator<list_node_buffer> buffer_allocator;
	 static buffer_pointer buffer_list;
    static link_type free_list;
    static link_type next_avail;
    static link_type last;
    void add_new_buffer() {
	buffer_pointer tmp = buffer_allocator.allocate((size_type)1);
	tmp->buffer = list_node_allocator.allocate(buffer_size());
	tmp->next_buffer = buffer_list;
	buffer_list = tmp;
	next_avail = buffer_list->buffer;
	last = next_avail + buffer_size();
	 }
	 static size_type number_of_lists;
	 void deallocate_buffers();
	 link_type get_node() {
	link_type tmp = free_list;
	return free_list ? (free_list = (link_type)(free_list->next), tmp)
		 : (next_avail == last ? (add_new_buffer(), next_avail++)
		: next_avail++);
	// ugly code for inlining - avoids multiple returns
	 }
	 void put_node(link_type p) {
	p->next = free_list;
	free_list = p;
	 }

protected:
	 link_type node;
	 size_type length;
public:
	 class iterator;
	 class const_iterator;
// CSH begin
			 class iterator_base
			 {
          protected:
          	 typedef list<T> owner_type;
          private:
				 iterator_base* _next_iter;
				 owner_type* _owner;
				 link_type node;

				 friend class iterator;
				 friend class const_iterator;
             friend class list<T>;
			 protected:
				 iterator_base() : _owner(0) {}
				 bool valid() const
				 {  return _owner != 0 && !past_end();
				 }
				 bool past_end() const
				 {  return _owner != 0 && node == _owner->node;
				 }
				 bool orphan() const
				 {  return _owner == 0;
				 }
				 bool same_owner(const iterator_base& b) const
				 {  if (orphan() || b.orphan()) return false;
					 return _owner == b._owner;
				 }
				 void attach(link_type n, owner_type* owner)
				 {  node = n;  _owner = owner;
					 // add into iterator chain of owner
					 _next_iter = _owner->_iter_chain;
					 _owner->_iter_chain = this;
				 }

				 void detach()
				 {  // remove from iterator chain of owner
					 if (_owner)
					 {  iterator_base* p = _owner->_iter_chain;
						 iterator_base* ppre = 0;
						 while (p != 0 && p != this)
						 {  ppre = p;
							 p = p->_next_iter;
						 }
						 if (ppre != 0)
							 ppre->_next_iter = p->_next_iter;
						 else
							 _owner->_iter_chain = p->_next_iter;
						 _owner = 0;
					 }
				 }

		 };
		 friend class iterator_base;
// CSH end

	 class iterator : public bidirectional_iterator<T, difference_type>,
	 private iterator_base // CSH
	  {
	 friend class list<T>;
	 friend class const_iterator;
//  friend bool operator==(const iterator& x, const iterator& y);
	 protected:
// CSH	link_type node;
// CSH	iterator(link_type x) : node(x) {}
	 public:
	iterator() {}
// CSH begin
		 private:
			 iterator(link_type x, owner_type* owner)
			 {  attach(x, owner);
			 }
		 public:
			 iterator(const iterator& b)
			 {  attach(b.node, b._owner);
			 }

			 iterator& operator=(const iterator& b)
			 {  detach();
				 attach(b.node, b._owner);
				 return *this;
			 }

			 ~iterator()
			 {  detach();
			 }
// CSH end

	bool operator==(const iterator& x) const {
		assert(!orphan() && !x.orphan()); // CSH
		return node == x.node; }
	reference operator*() const { assert(valid()); /* CSH */ return (*node).data; }
	iterator& operator++() {
		 assert(valid()); // CSH
		 node = (link_type)((*node).next);
		 return *this;
	}
	iterator operator++(int) {
		 assert(valid()); // CSH
		 iterator tmp = *this;
		 ++*this;
		 return tmp;
	}
	iterator& operator--() {
		 assert(!orphan()); // CSH
		 node = (link_type)((*node).prev);
		 assert(valid()); // CSH
		 return *this;
	}
	iterator operator--(int) {
		 assert(!orphan()); // CSH
		 iterator tmp = *this;
		 --*this;
		 assert(valid()); // CSH
		 return tmp;
	}
	 };
	 class const_iterator : public bidirectional_iterator<T, difference_type>,
	 private iterator_base // CSH
	  {
	 friend class list<T>;
	 protected:
// CSH	link_type node;
// CSH	const_iterator(link_type x) : node(x) {}
	 public:
	const_iterator() {}
// CSH begin
		 private:
			 const_iterator(link_type x, const owner_type* owner)
			 {  attach(x, (/*non-const*/owner_type*)owner);
			 }
		 public:
			 const_iterator(const const_iterator& b)
			 {  attach(b.node, b._owner);
			 }

			 const_iterator& operator=(const const_iterator& b)
			 {  detach();
				 attach(b.node, b._owner);
				 return *this;
			 }

			 ~const_iterator()
			 {  detach();
			 }
// CSH end

	const_iterator(const iterator& x) { attach(x.node, x._owner ); } // CSH
	bool operator==(const const_iterator& x) const {
		assert(!orphan() && !x.orphan()); // CSH
		return node == x.node; }
	const_reference operator*() const { assert(valid()); /* CSH */ return (*node).data; }
	const_iterator& operator++() {
		 assert(valid()); // CSH
		 node = (link_type)((*node).next);
		 return *this;
	}
	const_iterator operator++(int) {
		 assert(valid()); // CSH
		 const_iterator tmp = *this;
		 ++*this;
		 return tmp;
	}
	const_iterator& operator--() {
		 assert(!orphan()); // CSH
		 node = (link_type)((*node).prev);
		 assert(valid()); // CSH
		 return *this;
	}
	const_iterator operator--(int) {
		 assert(!orphan()); // CSH
		 const_iterator tmp = *this;
		 --*this;
		 assert(valid()); // CSH
		 return tmp;
	}
	 };
	 typedef reverse_bidirectional_iterator<const_iterator, value_type,
														 const_reference, difference_type>
	const_reverse_iterator;
	 typedef reverse_bidirectional_iterator<iterator, value_type, reference,
														 difference_type>
		  reverse_iterator;
// CSH begin
private:
			 iterator_base* _iter_chain;
			 void orphan_iterators()
			 {  iterator_base* p = _iter_chain;
				 _iter_chain = 0;
				 while (p != 0)
				 {  p->_owner = 0;
					 iterator_base* pnext = p->_next_iter;
					 p->_next_iter = 0;
					 p = pnext;
				 }
			 }
			 void orphan_iterators_at(iterator_base i)
			 {  iterator_base* p = _iter_chain;
				 iterator_base* pprev = 0;
				 while (p != 0)
				 {  iterator_base* pnext = p->_next_iter;
					 if (p->node == i.node)
					 {  p->_owner = 0;
						 p->_next_iter = 0;
						 if (pprev) pprev->_next_iter = pnext;
						 else _iter_chain = pnext;
					 }
					 else
						 pprev = p;
					 p = pnext;
				 }
			 }
			 bool owns(iterator_base p)
			 {	 return p._owner == this;
			 }

public:
// CSH end

	 list() : length(0), _iter_chain(0) /* CSH */ {
	++number_of_lists;
	node = get_node();
	(*node).next = node;
	(*node).prev = node;
	 }
	 iterator begin() { return iterator((link_type)((*node).next), this) /* CSH */; }
	 const_iterator begin() const { return const_iterator((link_type)((*node).next), this) /* CSH */; }
	 iterator end() { return iterator(node, this) /* CSH */; }
	 const_iterator end() const { return const_iterator(node, this) /* CSH */; }
	 reverse_iterator rbegin() { return reverse_iterator(end()); }
	 const_reverse_iterator rbegin() const {
		  return const_reverse_iterator(end());
	 }
	 reverse_iterator rend() { return reverse_iterator(begin()); }
	 const_reverse_iterator rend() const {
		  return const_reverse_iterator(begin());
	 }
	 bool empty() const { return length == 0; }
	 size_type size() const { return length; }
	 size_type max_size() const { return list_node_allocator.max_size(); }
	 reference front() { return *begin(); }
	 const_reference front() const { return *begin(); }
	 reference back() { return *(--end()); }
	 const_reference back() const { return *(--end()); }
	 void swap(list<T>& x) {
	 orphan_iterators(); // CSH
	 x.orphan_iterators(); // CSH
/* CSH */ std 	::swap(node, x.node);
/* CSH */ std 	::swap(length, x.length);
	 }
	 iterator insert(iterator position, const T& x) {
	 assert(owns(position)); // CSH
	link_type tmp = get_node();
	construct(value_allocator.address((*tmp).data), x);
	(*tmp).next = position.node;
	(*tmp).prev = (*position.node).prev;
	(*(link_type((*position.node).prev))).next = tmp;
	(*position.node).prev = tmp;
	++length;
	return iterator(tmp, this) /* CSH */;
	 }
	 void insert(iterator position, const T* first, const T* last);
	 void insert(iterator position, const_iterator first,
		const_iterator last);
	 void insert(iterator position, size_type n, const T& x);
	 void push_front(const T& x) { insert(begin(), x); }
	 void push_back(const T& x) { insert(end(), x); }
	 void erase(iterator position) {
	 assert(owns(position)); // CSH
	(*(link_type((*position.node).prev))).next = (*position.node).next;
	(*(link_type((*position.node).next))).prev = (*position.node).prev;
	orphan_iterators_at(position); // CSH
	destroy(value_allocator.address((*position.node).data));
	put_node(position.node);
	--length;
	 }
	 void erase(iterator first, iterator last);
	 void pop_front() { erase(begin()); }
	 void pop_back() {
	iterator tmp = end();
	erase(--tmp);
	 }
	 list(size_type n, const T& value = T()) : length(0), _iter_chain(0) /* CSH */ {
	++number_of_lists;
	node = get_node();
	(*node).next = node;
	(*node).prev = node;
	insert(begin(), n, value);
	 }
	 list(const T* first, const T* last) : length(0), _iter_chain(0) /* CSH */ {
	++number_of_lists;
	node = get_node();
	(*node).next = node;
	(*node).prev = node;
	insert(begin(), first, last);
	 }
	 list(const list<T>& x) : length(0), _iter_chain(0) /* CSH */ {
	++number_of_lists;
	node = get_node();
	(*node).next = node;
	(*node).prev = node;
	insert(begin(), x.begin(), x.end());
	 }
	 ~list() {
	erase(begin(), end());
	put_node(node);
	if (--number_of_lists == 0) deallocate_buffers();
	 }
	 list<T>& operator=(const list<T>& x);
protected:
	 void transfer(iterator position, iterator first, iterator last) {
	(*(link_type((*last.node).prev))).next = position.node;
	(*(link_type((*first.node).prev))).next = last.node;
	(*(link_type((*position.node).prev))).next = first.node;
	link_type tmp = link_type((*position.node).prev);
	(*position.node).prev = (*last.node).prev;
	(*last.node).prev = (*first.node).prev;
	(*first.node).prev = tmp;
	 }
public:
	 void splice(iterator position, list<T>& x) {
	 assert(owns(position)); // CSH
	if (!x.empty()) {
		 transfer(position, x.begin(), x.end());
		 x.orphan_iterators(); // CSH
		 length += x.length;
		 x.length = 0;
	}
	 }
	 void splice(iterator position, list<T>& x, iterator i) {
	 assert(owns(position)); // CSH
	 assert(x.owns(i)); // CSH
	iterator j = i;
	if (position == i || position == ++j) return;
	transfer(position, i, j);
		 x.orphan_iterators(); // CSH
	++length;
	--x.length;
	 }
	 void splice(iterator position, list<T>& x, iterator first, iterator last) {
	 assert(owns(position)); // CSH
	 assert(x.owns(first) && x.owns(last)); // CSH
	if (first != last) {
		 if (&x != this) {
		difference_type n = 0;
			distance(first, last, n);
			x.length -= n;
			length += n;
		 }
		 transfer(position, first, last);
		 x.orphan_iterators();
	}
	 }
	 void remove(const T& value);
	 void unique();
	 void merge(list<T>& x);
	 void reverse();
	 void sort();
};

template <class T>
list<T>::buffer_pointer list<T>::buffer_list = 0;

template <class T>
list<T>::link_type list<T>::free_list = 0;

template <class T>
list<T>::link_type list<T>::next_avail = 0;

template <class T>
list<T>::link_type list<T>::last = 0;

template <class T>
list<T>::size_type list<T>::number_of_lists = 0;

template <class T>
list<T>::list_node_allocator_type list<T>::list_node_allocator;

template <class T>
list<T>::value_allocator_type list<T>::value_allocator;

template <class T>
list<T>::buffer_allocator_type list<T>::buffer_allocator;

/*
 * currently the following does not work - made into a member function

template <class T>
inline bool operator==(const list<T>::iterator& x, const list<T>::iterator& y) {
	 return x.node == y.node;
}
*/

template <class T>
inline bool operator==(const list<T>& x, const list<T>& y) {
	 return x.size() == y.size() && equal(x.begin(), x.end(), y.begin());
}

template <class T>
inline bool operator<(const list<T>& x, const list<T>& y) {
	 return lexicographical_compare(x.begin(), x.end(), y.begin(), y.end());
}

template <class T>
void list<T>::deallocate_buffers() {
	 while (buffer_list) {
	buffer_pointer tmp = buffer_list;
	buffer_list = (buffer_pointer)(buffer_list->next_buffer);
	list_node_allocator.deallocate(tmp->buffer);
	buffer_allocator.deallocate(tmp);
	 }
	 free_list = 0;
	 next_avail = 0;
	 last = 0;
}

template <class T>
void list<T>::insert(iterator position, const T* first, const T* last) {
	assert(owns(position)); // CSH
	 while (first != last) insert(position, *first++);
}

template <class T>
void list<T>::insert(iterator position, const_iterator first,
			  const_iterator last) {
	assert(owns(position)); // CSH
	assert(first.same_owner(last)); // CSH
	 while (first != last) insert(position, *first++);
}

template <class T>
void list<T>::insert(iterator position, size_type n, const T& x) {
	assert(owns(position)); // CSH
	 while (n--) insert(position, x);
}

template <class T>
void list<T>::erase(iterator first, iterator last) {
	assert(owns(first) && owns(last)); // CSH
	 while (first != last) erase(first++);
}

template <class T>
list<T>& list<T>::operator=(const list<T>& x) {
	 if (this != &x) {
	iterator first1 = begin();
	iterator last1 = end();
	const_iterator first2 = x.begin();
	const_iterator last2 = x.end();
	while (first1 != last1 && first2 != last2) *first1++ = *first2++;
	if (first2 == last2)
		 erase(first1, last1);
	else
		 insert(last1, first2, last2);
	orphan_iterators(); // CSH
	 }
	 return *this;
}

template <class T>
void list<T>::remove(const T& value) {
	 iterator first = begin();
	 iterator last = end();
	 while (first != last) {
	iterator next = first;
	++next;
	if (*first == value) erase(first);
	first = next;
	 }
}

template <class T>
void list<T>::unique() {
	 iterator first = begin();
	 iterator last = end();
	 if (first == last) return;
	 iterator next = first;
	 while (++next != last) {
	if (*first == *next)
		 erase(next);
	else
		 first = next;
	next = first;
	 }
}

template <class T>
void list<T>::merge(list<T>& x) {
	 iterator first1 = begin();
	 iterator last1 = end();
	 iterator first2 = x.begin();
	 iterator last2 = x.end();
	 while (first1 != last1 && first2 != last2)
	if (*first2 < *first1) {
		 iterator next = first2;
		 transfer(first1, first2, ++next);
		 first2 = next;
	} else
		 ++first1;
	 if (first2 != last2) transfer(last1, first2, last2);
	 length += x.length;
	 x.length= 0;
	 x.orphan_iterators(); // CSH
}

template <class T>
void list<T>::reverse() {
	 if (size() < 2) return;
	 for (iterator first = ++begin(); first != end();) {
	iterator old = first++;
	transfer(begin(), old, first);
	 }
	 orphan_iterators(); // CSH
}

template <class T>
void list<T>::sort() {
	 if (size() < 2) return;
	 list<T> carry;
	 list<T> counter[64];
	 int fill = 0;
	 while (!empty()) {
	carry.splice(carry.begin(), *this, begin());
	int i = 0;
	while(i < fill && !counter[i].empty()) {
		 counter[i].merge(carry);
		 carry.swap(counter[i++]);
	}
	carry.swap(counter[i]);
	if (i == fill) ++fill;
	 }
	 while(fill--) merge(counter[fill]);
	 orphan_iterators(); // CSH
}

#undef Allocator
#undef list

// CSH begin
} // namespace std
// CSH end

#endif
